!!ARBfp1.0

ATTRIB iColor = fragment.color.primary;
ATTRIB iCoord = fragment.texcoord;
ATTRIB iPosition = fragment.position;

PARAM desc0 = program.local[0]; # x: relative X; y: relative Y; z: refract scale; w: reflect alpha;
PARAM desc1 = program.local[1]; # x: specular alpha; y: shine; z: bump X scale; w: bump Y scale;
PARAM desc2 = program.local[2]; # x: texture X scale; y: texture Y scale; z: reflect X scale; w: reflect Y scale;
PARAM lightvec = program.local[3]; # xyz: light vector; w: shadow alpha;
PARAM texrel = program.local[4]; # xyzw: relative texture coords;
PARAM bumprel = program.local[5]; # xyzw: relative bump coords;
PARAM refrel = program.local[6]; # xyzw: relative reflect coords;
PARAM maskrel = program.local[7]; # xyzw: relative mask coords;

OUTPUT oColor = result.color;

#---------#
# Program #
#---------#

TEMP light;
MOV light, lightvec;
DP3 light.w, light, light;
RSQ light.w, light.w;
MUL light.xyz, light, light.w;
TEMP eyevec;
MOV eyevec.xy, 0.0;
MOV eyevec.z, 1.0;
TEMP normal;
MOV normal.xy, 0.0;
MOV normal.z, 1.0;
TEMP pos;

# mask
ADD pos, iCoord, maskrel;
TEMP mask;
TXP mask, pos, texture[3], 2D;
TEMP test;
CMP test, -mask.a, 1.0, -1.0;
KIL test;

# normal map
ADD pos, iCoord, desc0;
ADD pos, pos, bumprel;
MUL pos.x, pos.x, desc1.z;
MUL pos.y, pos.y, desc1.w;
MOV pos.w, 1.0;
TEMP bump;
TXP bump, pos, texture[1], 2D;
MUL bump.x, bump.x, 2.0;
MUL bump.y, bump.y, 2.0;
ADD bump.x, bump.x, -1.0;
ADD bump.y, bump.y, -1.0;
MOV bump.y, -bump.y;
TEMP temp;
MOV temp, 0.0;
MOV temp.x, bump.x;
MOV temp.y, bump.y;
DP3 temp, temp, temp;
MOV bump.z, temp.z;
ADD bump.z, 1.0, -bump.z;
RSQ bump.z, bump.z;
RCP bump.z, bump.z;

# attenuation
TEMP att;
MAX att, 0.0, light.z;
ADD att, 1.0, -att;
MUL att, att, att;
ADD att, 1.0, -att;

# diffuse intensivity
TEMP diffint;
DP3 diffint, light, bump;
MOV_SAT diffint, diffint;

# specular intensivity
TEMP h;
ADD h, light, eyevec;
DP3 h.w, h, h;
RSQ h.w, h.w;
MUL h.xyz, h, h.w;
TEMP specint;
DP3 specint, h, bump;
MOV_SAT specint, specint;
POW specint, specint.w, desc1.y;

# refract
TEMP refract;
DP3 refract, normal, bump;
MOV_SAT refract, refract;
MUL pos, bump, refract;
MUL pos, pos, desc0.z;
RCP h.x, desc1.z;
RCP h.y, desc1.w;
MUL pos.x, pos.x, h.x;
MUL pos.y, pos.y, h.y;
ADD pos, iCoord, pos;

# diffuse color
MOV h, pos;
ADD pos, pos, texrel;
MUL pos.x, pos.x, desc2.x;
MUL pos.y, pos.y, desc2.y;
TEMP samp;
TXP samp, pos, texture[0], 2D;
TEMP out;
MUL out, samp, iColor;
TEMP src;
MOV src, out;
MOV pos, h;

# reflect color
ADD pos, pos, refrel;
MUL pos.x, pos.x, desc2.z;
MUL pos.y, pos.y, desc2.w;
TXP samp, pos, texture[2], 2D;
TEMP refl;
MUL refl, samp, iColor;

# mix diffuse & reflect
MUL refl, refl, desc0.w;
TEMP tmp;
ADD tmp, 1.0, -desc0.w;
MUL out, out, tmp;
ADD out, refl, out;
MOV pos, out;

# result
MUL out, pos, diffint;
MUL out, out, att;
MUL out, out, lightvec.w;
ADD tmp, 1.0, -lightvec.w;
MUL pos, pos, tmp;
ADD out, pos, out;
MUL h, specint, desc1.x;
ADD out, out, h;
MOV out.a, src.a;
MUL out, out, mask;

# set output
MOV oColor, out;

END